/* -*- C++ -*-
    This file is part of ThreadWeaver.

    SPDX-FileCopyrightText: 2005-2013 Mirko Boehm <mirko@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef QUEUESIGNALS_P_H
#define QUEUESIGNALS_P_H

namespace ThreadWeaver
{
namespace Private
{
class QueueSignals_Private
{
public:
    QueueSignals_Private();
    virtual ~QueueSignals_Private();
};

}
}

#endif // QUEUESIGNALS_P_H
