/* -*- C++ -*-
    Shared pointer based jobs in ThreadWeaver.

    SPDX-FileCopyrightText: 2005-2013 Mirko Boehm <mirko@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef THREADWEAVER_JOBPOINTER_H
#define THREADWEAVER_JOBPOINTER_H

// JobPointer is declared together with JobInterface
#include "jobinterface.h"

#endif // THREADWEAVER_JOBPOINTER_H
