/* -*- C++ -*-
    This file implements the public interfaces of the Weaver and the Job class.
    It should be the only include file necessary to use the ThreadWeaver
    library.

    SPDX-FileCopyrightText: 2005-2013 Mirko Boehm <mirko@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/
#ifndef THREADWEAVER_H
#define THREADWEAVER_H

#include "collection.h"
#include "job.h"
#include "jobinterface.h"
#include "jobpointer.h"
#include "queue.h"
#include "queueing.h"
#include "sequence.h"

#endif // THREADWEAVER_H
